unit unMainForm;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, ActnList, IBCtrls, Spin;

type
  TMainForm = class(TForm)
    OpenDataBaseDialog: TOpenDialog;
    lbxLog: TListBox;
    pnLog: TPanel;
    btOpenDatabase: TButton;
    btForceClearLogs: TButton;
    btForceSetNewGen: TButton;
    ActionList1: TActionList;
    actOpenDatabase: TAction;
    actActivate: TAction;
    cbxActivate: TCheckBox;
    actForceSetNewGen: TAction;
    actForceClearLogs: TAction;
    pnControls: TPanel;
    sedTrySetNewGenInterval: TSpinEdit;
    sedClearLogsInterval: TSpinEdit;
    sedMaxSetNewGenTries: TSpinEdit;
    sedMaxChangesCount: TSpinEdit;
    lbTrySetNewGenInterval: TLabel;
    lbClearLogsInterval: TLabel;
    lbMaxSetNewGenTries: TLabel;
    lbMaxChangesCount: TLabel;
    procedure FormShow(Sender: TObject);
    procedure actOpenDatabaseExecute(Sender: TObject);
    procedure actActivateExecute(Sender: TObject);
    procedure actForceClearLogsExecute(Sender: TObject);
    procedure actForceSetNewGenExecute(Sender: TObject);
    procedure sedTrySetNewGenIntervalChange(Sender: TObject);
    procedure sedClearLogsIntervalChange(Sender: TObject);
    procedure sedMaxSetNewGenTriesChange(Sender: TObject);
    procedure sedMaxChangesCountChange(Sender: TObject);
  private
    procedure BackgroundsDMAfterClearLogs(Sender: TObject);
    procedure BackgroundsDMAfterTrySetNewGen(Sender: TObject; ChangesCount: Integer );
    procedure RefreshCaption;
    procedure AddLog( const Str: string );
    procedure AddLogFmt( const FormatStr: string; const Args: array of TVarRec  ); 
    { Private declarations }
  protected
    procedure WndProc( var Message: TMessage ); override;
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

uses unBackgroundsDM, unGlobal;

{$R *.dfm}
procedure TMainForm.BackgroundsDMAfterTrySetNewGen( Sender: TObject; ChangesCount: Integer );
  begin
    AddLogFmt( 'TrySetNewGen %d', [ BackgroundsDM.GetChangesCount ] );
    RefreshCaption;
  end;

procedure TMainForm.BackgroundsDMAfterClearLogs( Sender: TObject );
  begin
    AddLog( 'ClearLogs' );
    RefreshCaption;
  end;

procedure TMainForm.FormShow( Sender: TObject );
  begin
    BackgroundsDM.OnAfterTrySetNewGen := BackgroundsDMAfterTrySetNewGen;
    BackgroundsDM.OnAfterClearLogs    := BackgroundsDMAfterClearLogs;
  end;

procedure TMainForm.RefreshCaption;
 var
   CurrDBID: Integer;
   CurrGen:  Integer;
   DatabaseName: string;
 begin
   CurrDBID :=  BackgroundsDM.GetCurrDBID;
   CurrGen  :=  BackgroundsDM.GetCurrGen;
   DatabaseName := BackgroundsDM.DatabaseName;
   Caption := Format( 'DB: %d GEN: %d [%s]', [ CurrDBID, CurrGen, DatabaseName ] );
 end;


procedure TMainForm.actOpenDatabaseExecute(Sender: TObject);
  begin
    if OpenDataBaseDialog.Execute then
      begin
        BackgroundsDM.Open( OpenDataBaseDialog.FileName, False );
        actActivate.Execute;
        sedTrySetNewGenInterval.Value  := BackgroundsDM.TrySetNewGenInterval div 1000;
        sedClearLogsInterval.Value     := BackgroundsDM.ClearLogsInterval div 1000;
        sedMaxSetNewGenTries.Value     := BackgroundsDM.MaxSetNewGenTries;
        sedMaxChangesCount.Value       := BackgroundsDM.MaxChangesCount;
        RefreshCaption;
      end;
  end;

procedure TMainForm.actActivateExecute(Sender: TObject);
  begin
    BackgroundsDM.Active := actActivate.Checked;
    if BackgroundsDM.Active then
      AddLog( '--activated--' )
    else
      AddLog( '--deactivated--' );
  end;

procedure TMainForm.actForceClearLogsExecute(Sender: TObject);
  begin
    BackgroundsDM.ClearLogs;
  end;

procedure TMainForm.actForceSetNewGenExecute(Sender: TObject);
  begin
    BackgroundsDM.TrySetNewGen( True );
  end;

procedure TMainForm.AddLog(const Str: string);
  var
    i: Integer;
  begin
    lbxLog.Items.Insert( 0, '[' + TimeToStr( Now ) + '] ' + Str );
    lbxLog.ItemIndex := 0;
    if lbxLog.Items.Count > 2000 then
      for i := lbxLog.Items.Count - 1 downto lbxLog.Items.Count div 2 do
        lbxLog.Items.Delete( i );
  end;

procedure TMainForm.AddLogFmt( const FormatStr: string; const Args: array of TVarRec );
  begin
    AddLog( Format( FormatStr, Args ) );
  end;

procedure TMainForm.sedTrySetNewGenIntervalChange(Sender: TObject);
  begin
    BackgroundsDM.TrySetNewGenInterval := sedTrySetNewGenInterval.Value * 1000;
  end;

procedure TMainForm.sedClearLogsIntervalChange(Sender: TObject);
  begin
    BackgroundsDM.ClearLogsInterval := sedClearLogsInterval.Value * 1000;
  end;

procedure TMainForm.sedMaxSetNewGenTriesChange(Sender: TObject);
  begin
    BackgroundsDM.MaxSetNewGenTries := sedMaxSetNewGenTries.Value;
  end;

procedure TMainForm.sedMaxChangesCountChange(Sender: TObject);
  begin
    BackgroundsDM.MaxChangesCount := sedMaxChangesCount.Value;
  end;

procedure TMainForm.WndProc( var Message: TMessage );
  begin
    if  Message.Msg = ForceSetNewGenMsg then
      if BackgroundsDM.Opened then
        if Message.LParam = BackgroundsDM.GetCurrDBID then
          BackgroundsDM.TrySetNewGen( True );
    inherited;
  end;

end.
