unit unClientMainForm;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, DB, Grids, DBGrids, ExtCtrls, DBCtrls, ActnList, Menus,
  IBCustomDataSet, IBQuery, ComCtrls, IBDatabase;

type
  TClientMainForm = class(TForm)
    DetailDS: TDataSource;
    MasterDS: TDataSource;
    DBProfileDS: TDataSource;
    nvMaster: TDBNavigator;
    MainMenu: TMainMenu;
    ActionList: TActionList;
    actRefreshAll: TAction;
    miRefreshAll: TMenuItem;
    actSetNewGeneration: TAction;
    miSetNewGeneration: TMenuItem;
    actDeleteIsDeleted: TAction;
    DeleteIsDeleted1: TMenuItem;
    actReceive: TAction;
    actSend: TAction;
    miReceive: TMenuItem;
    LogMasterDS: TDataSource;
    actOpen: TAction;
    miOpen: TMenuItem;
    OpenDbDialog: TOpenDialog;
    LogDetailDS: TDataSource;
    PageControl: TPageControl;
    tshMaster: TTabSheet;
    grLogMaster: TDBGrid;
    grMaster: TDBGrid;
    spltMaster: TSplitter;
    tshDetail: TTabSheet;
    grLogDetail: TDBGrid;
    grDetail: TDBGrid;
    spltDetail: TSplitter;
    nvDetail: TDBNavigator;
    miTest1: TMenuItem;
    TestQuery: TIBQuery;
    TestTransaction: TIBTransaction;
    miNew: TMenuItem;
    actNew: TAction;
    SaveNewDBAsDialog: TSaveDialog;
    OpenGenDBIDDialog: TOpenDialog;
    OpenServerDBDialog: TOpenDialog;
    actDeleteDB: TAction;
    miDeleteDB: TMenuItem;
    ClientDS: TDataSource;
    LogDBProfileDS: TDataSource;
    tshDB: TTabSheet;
    Panel1: TPanel;
    Panel2: TPanel;
    Splitter1: TSplitter;
    gbxClient: TGroupBox;
    gbxLogDBProfile: TGroupBox;
    gbxDBProfile: TGroupBox;
    grClient: TDBGrid;
    Splitter2: TSplitter;
    grDBProfile: TDBGrid;
    grLogDBProfile: TDBGrid;
    gbxLogMaster: TGroupBox;
    gbxMaster: TGroupBox;
    gbxLogDetail: TGroupBox;
    gbxDetail: TGroupBox;
    procedure actRefreshAllExecute(Sender: TObject);
    procedure actReceiveExecute(Sender: TObject);
    procedure actSetNewGenerationExecute(Sender: TObject);
    procedure actDeleteIsDeletedExecute(Sender: TObject);
    procedure actOpenExecute(Sender: TObject);
    procedure miTest1Click(Sender: TObject);
    procedure actNewExecute(Sender: TObject);
    procedure actDeleteDBExecute(Sender: TObject);
    procedure grDetailEditButtonClick(Sender: TObject);
    procedure grLogDataDrawDataCell(Sender: TObject; const Rect: TRect;
      Field: TField; State: TGridDrawState);
    procedure grLogDBProfileDrawDataCell(Sender: TObject;
      const Rect: TRect; Field: TField; State: TGridDrawState);
    procedure FormShow(Sender: TObject);
  private
    { Private declarations }
    procedure RefreshCaption;
    procedure RemoteDMBeforeCommitReceiveGen(Sender: TObject;
      Gen: Integer);
  public
    { Public declarations }
  end;

var
  ClientMainForm: TClientMainForm;

implementation

uses unClientDataDM,  unClientMainDM,  unBackgroundDM, unInteractionDM, unTools,
  unTestForm, unGenDBIDDM, unSelectMasterForm, unServerChangesForm;
{$R *.dfm}


procedure TClientMainForm.actRefreshAllExecute(Sender: TObject);
  begin
    RefreshCaption;
    ClientDataDM.RefreshAll;
  end;

procedure TClientMainForm.actReceiveExecute(Sender: TObject);
  begin
    if OpenServerDBDialog.Execute then
      begin
        InteractionDM.Receive( OpenServerDBDialog.FileName );
        RefreshCaption;
        ClientDataDM.RefreshAll;
      end;
  end;

procedure TClientMainForm.RemoteDMBeforeCommitReceiveGen(Sender: TObject;  Gen: Integer);
  begin
    if ServerChangesForm.ShowModalParams( gen ) = mrCancel then
      ABORT;
  end;



procedure TClientMainForm.actNewExecute(Sender: TObject);
  var
    DBID: Integer;
  begin
   if OpenGenDBIDDialog.Execute then
      begin
        GenDBIDDM.Open( OpenGenDBIDDialog.FileName );
        try
          DBID := GenDBIDDM.GetNewDBID;
        finally
          GenDBIDDM.Close;
        end;
        SaveNewDBAsDialog.FileName := Format( 'db%d.gdb', [ DBID ] );
        if SaveNewDBAsDialog.Execute then
          begin
            ClientMainDM.CreateNewDB( SaveNewDBAsDialog.FileName, DBID );
            RefreshCaption;
            ClientDataDM.RefreshAll;
          end;
      end;
  end;


procedure TClientMainForm.actDeleteDBExecute(Sender: TObject);
  begin
    ClientMainDM.DeleteCurrDB;
    RefreshCaption;
    ClientDataDM.RefreshAll;
  end;



procedure TClientMainForm.actSetNewGenerationExecute(Sender: TObject);
  begin
    BackgroundDM.SetNewGeneration;
    RefreshCaption;
    ClientDataDM.RefreshAll;
  end;

procedure TClientMainForm.actDeleteIsDeletedExecute(Sender: TObject);
  begin
    BackgroundDM.ClearLogs;
    RefreshCaption;
    ClientDataDM.RefreshAll;
  end;

procedure TClientMainForm.actOpenExecute(Sender: TObject);
  begin
    if OpenDBDialog.Execute then
      begin
        ClientMainDM.Open( OpenDBDialog.FileName );
        RefreshCaption;
        ClientDataDM.RefreshAll;
      end;
  end;

procedure TClientMainForm.miTest1Click(Sender: TObject);
  begin
    TestForm.Show;
  end;

procedure TClientMainForm.RefreshCaption;
 var
   CurrDBID: Integer;
   CurrGen:  Integer;
   DatabaseName: string;
 begin
   CurrDBID :=  ClientMainDM.GetCurrDBID;
   CurrGen  :=  ClientMainDM.GetCurrGen;
   DatabaseName := ClientMainDM.DatabaseName;
   Caption := Format( 'DB: %d GEN: %d [%s]', [ CurrDBID, CurrGen, DatabaseName ] );
 end;

procedure TClientMainForm.grDetailEditButtonClick( Sender: TObject );
  begin
    if grDetail.SelectedField.FieldName = 'MASTER_GID' then
      begin
        DetailDS.DataSet.Edit;
        SelectMasterForm.ShowModal;
      end;
  end;

procedure TClientMainForm.grLogDataDrawDataCell(Sender: TObject;
  const Rect: TRect; Field: TField; State: TGridDrawState);
  begin
    with Sender as TDBGrid, Canvas do
      if not DataSource.DataSet.IsEmpty then
        begin
          if DataSource.DataSet.FieldByName( 'LOG_UPD_GEN' ).IsNull then
            Brush.Color := clMoneyGreen;

          if DataSource.DataSet.FieldByName( 'LOG_IS_DELETED' ).AsInteger <> 0 then
            if ( Field.FieldName  = 'LOG_GID' ) or ( Field.FieldName  = 'LOG_IS_DELETED' ) then
              Font.Color := clRed;

          DefaultDrawDataCell( Rect, Field, State );
        end;
  end;

procedure TClientMainForm.grLogDBProfileDrawDataCell(Sender: TObject;
  const Rect: TRect; Field: TField; State: TGridDrawState);
  begin
    with Sender as TDBGrid, Canvas do
      if not DataSource.DataSet.IsEmpty then
        begin
          if DataSource.DataSet.FieldByName( 'LOG_UPD_GEN' ).IsNull then
              Brush.Color := clMoneyGreen;

          if DataSource.DataSet.FieldByName( 'LOG_IS_DELETED' ).AsInteger <> 0 then
            if ( Field.FieldName  = 'LOG_DB_ID' ) or ( Field.FieldName  = 'LOG_IS_DELETED' ) then
              Font.Color := clRed;

          if DataSource.DataSet.FieldByName( 'LOG_IS_CURR' ).AsInteger = 1 then
            Font.Style := Font.Style + [ fsBold ];

          DefaultDrawDataCell( Rect, Field, State );
        end;
  end;

procedure TClientMainForm.FormShow(Sender: TObject);
  begin
    InteractionDM.OnBeforeCommitReceiveGen := RemoteDMBeforeCommitReceiveGen;
  end;

end.
