(*********************************************************)
(* TClientMainDM                 *)
(*    ()                  *)
(*    			 *)
(*  ClientMainDM.Database          *)
(*    Database                        *)
(*  DataModule     *)
(*********************************************************)
unit unClientMainDM;

interface

uses
  SysUtils, Classes, IBCustomDataSet, IBUpdateSQL, DB, IBTable, IBDatabase,
  IBQuery, IBStoredProc;

type
  TClientMainDM = class(TDataModule)
    Database: TIBDatabase;
    Transaction: TIBTransaction;
    Query: TIBQuery;
    SetCurrDBProc: TIBStoredProc;
  private
    { Private declarations }
    procedure InsertDB(DBID: Integer);
    procedure SetCurrDB(DBID: Integer);
    function  GetDatabaseFileName: string;
  public
    { Public declarations }
    property  DatabaseName: string read GetDatabaseFileName;
    function  GetCurrGen: Integer;
    function  GetCurrDBID:Integer;
    function  GetServGen( DB_ID: Integer ):Integer;
    function  Deleted: Boolean;
    procedure Open( const FileName: string );
    procedure Close;
    procedure DeleteCurrDB;(*     (IS_CURR = 1)  DB_PROFILE*)
    procedure CreateNewDB( const NewDBFileName: string; DBID: Integer );(*    id = DBID*)
  end;

var
  ClientMainDM: TClientMainDM;

implementation

uses
  Windows;

{$R *.dfm}

const TemplateFileName: string = 'TEMPLATE.GDB';(*    "".       *)

function TClientMainDM.GetCurrGen: Integer;
  const                       
    SQL: string = 'select GEN from CURR_GEN';
  begin
    Query.SQL.Text := SQL;
    Transaction.StartTransaction;
    try
      Query.Open;
      try
        Query.First;
        Result := Query.FieldByName( 'GEN' ).AsInteger;
        Query.Next;
        if not Query.Eof then
           raise Exception.Create( 'TServerMainDM.GetCurrGen' );
      finally
        Query.Close;
      end;
      Transaction.Commit;
    except
      Transaction.Rollback;
      raise;
    end;
  end;

function TClientMainDM.GetCurrDBID: Integer;
  const
    SQL: string = 'select DB_ID from CURR_DB_ID';
  begin
    Query.SQL.Text := SQL;
    Transaction.StartTransaction;
    try
      Query.Open;
      try
        Query.First;
        Result := Query.FieldByName( 'DB_ID' ).AsInteger;
        Query.Next;
        if not Query.Eof then
            raise Exception.Create( 'TClientMainDM.GetCurrDB_ID' );
      finally
        Query.Close;
      end;
      Transaction.Commit;
    except
      Transaction.Rollback;
      raise;
    end;
  end;

function TClientMainDM.GetServGen( DB_ID: Integer ): Integer;
  const
    SQL: string = 'select SERV_GEN from DB_PROFILE where DB_ID = :DB_ID';
  begin
    Query.SQL.Text := SQL;
    Query.ParamByName( 'DB_ID' ).AsInteger := DB_ID;
    Transaction.StartTransaction;
    try
      Query.Open;
      try
        Query.First;
        Result := Query.FieldByName( 'SERV_GEN' ).AsInteger;
        Query.Next;
        if not Query.Eof then
            raise Exception.Create( 'TClientMainDM.GetMainUID' );
      finally
        Query.Close;
      end;
      Transaction.Commit;
    except
      Transaction.Rollback;
      raise;
    end;
  end;



function TClientMainDM.Deleted: Boolean;
  const
    SQL: string = 'select COUNT(*) as CURR_COUNT from DB_PROFILE where IS_CURR = 1';
  begin
    Query.SQL.Text := SQL;
    Transaction.StartTransaction;
    try
      Query.Open;
      try
        Query.First;
        Result := Query.FieldByName( 'CURR_COUNT' ).AsInteger = 0 ;
        Query.Next;
        if not Query.Eof then
            raise Exception.Create( 'TClientMainDM.Deleted' );
      finally
        Query.Close;
      end;
      Transaction.Commit;
    except
      Transaction.Rollback;
      raise;
    end;
  end;



procedure TClientMainDM.SetCurrDB( DBID: Integer );
  begin
    SetCurrDBProc.ParamByName( 'A_CURR_DB_ID' ).AsInteger := DBID;
    Transaction.StartTransaction;
    try
      SetCurrDBProc.ExecProc;
      Transaction.Commit;
    except
      Transaction.Rollback;
      raise;
    end;
  end;


procedure TClientMainDM.InsertDB( DBID: Integer );
  const
    SQL:string = 'insert into DB_PROFILE( DB_ID ) values ( :DB_ID )';
  begin
    Query.SQL.Text := SQL;
    Query.ParamByName( 'DB_ID' ).AsInteger := DBID;
    Transaction.StartTransaction;
    try
      Query.ExecSQL;
      Transaction.Commit;
    except
      Transaction.Rollback;
      raise;
    end;
  end;

procedure TClientMainDM.DeleteCurrDB;
  const
    SQL:string = 'delete from DB_PROFILE where IS_CURR = 1';
  begin
    Query.SQL.Text := SQL;
    Transaction.StartTransaction;
    try
      Query.ExecSQL;
      Transaction.Commit;
    except
      Transaction.Rollback;
      raise;
    end;
  end;





procedure TClientMainDM.Close;
  begin
    Database.Close;
  end;

procedure TClientMainDM.Open(const FileName: string);
  begin
    Database.Connected := false;
    Database.DatabaseName := FileName;
    Database.Open;
  end;



procedure TClientMainDM.CreateNewDB( const NewDBFileName: string; DBID: Integer );
  begin
    if not CopyFile( PChar(ExtractFilePath( ParamStr( 0 ) ) + TemplateFileName), PChar(NewDBFileName), False ) then
      raise Exception.CreateFmt( 'Cannot create %s, or cannot find %s',
                                 [ NewDBFileName, TemplateFileName ] );//     
    Open( NewDBFileName );

    SetCurrDB( DBID );
    InsertDB( DBID );//     ,      
  end;


function TClientMainDM.GetDatabaseFileName: string;
  begin
    Result := Database.DatabaseName;
  end;



end.
