(************************************************************)
(* TInteractionDM                          *)
(*     .                      *)
(*,                *)
(*  ServerMainDM.Database   Server_Transaction,    *)
(*     Server_ .                  *)
(*,                *)
(*  ClientMainDM.Database   Client_Transaction,    *)
(*     Client_ .                  *)
(*,  ClientMainDM                      *)
(*(  ClientMainDM.Open).                         *)
(*   ServerMainDM                        *)
(*( ServerMainDM.Open  ServerMainDM.Close ).       *)
(************************************************************)
unit unInteractionDM;

interface

uses
  SysUtils, Classes, DB, IBCustomDataSet, IBQuery, IBDatabase, IBTable,
  IBStoredProc;

type
  TReceiveGenEvent = procedure( Sender: TObject; Gen: Integer ) of object;

  TInteractionDM = class(TDataModule)
    Client_Transaction: TIBTransaction;
    Client_Query: TIBQuery;
    Client_BeginReceiveProc: TIBStoredProc;
    Client_EndReceiveProc: TIBStoredProc;
    Server_DetailGeneration: TIBQuery;
    Server_MasterGeneration: TIBQuery;
    Client_ProcessInputTablesProc: TIBStoredProc;
    Server_DBIDGeneration: TIBQuery;
    Server_Transaction: TIBTransaction;
    Client_MasterInput: TIBTable;
    Client_DetailInput: TIBTable;
    Client_DBProfileInput: TIBTable;
    Client_MasterInputLOG_GID: TLargeintField;
    Client_MasterInputLOG_OWN_GEN: TIntegerField;
    Client_MasterInputLOG_ACTION: TIBStringField;
    Client_MasterInputDATA: TIBStringField;
    Client_DetailInputLOG_GID: TLargeintField;
    Client_DetailInputLOG_OWN_GEN: TIntegerField;
    Client_DetailInputLOG_ACTION: TIBStringField;
    Client_DetailInputDATA: TIBStringField;
    Client_DetailInputMASTER_GID: TLargeintField;
    Client_DBProfileInputLOG_DB_ID: TIntegerField;
    Client_DBProfileInputLOG_ACTION: TIBStringField;
    procedure Client_MasterInputLOG_GIDGetText(Sender: TField;
      var Text: String; DisplayText: Boolean);
    procedure Client_DetailInputLOG_GIDGetText(Sender: TField;
      var Text: String; DisplayText: Boolean);
  private
    FAfterCommitReceiveGen: TReceiveGenEvent;
    FBeforeCommitReceiveGen: TReceiveGenEvent;
    procedure ReceiveMaster(  Gen: Integer );
    procedure ReceiveDetail(  Gen: Integer );
    procedure ReceiveDBID(    Gen: Integer );
    procedure ApplyReceivedData;
    procedure StartReceive( SERV_GEN: Integer );
    procedure CommitReceive;
    procedure RollbackReceive;
    procedure SetAfterCommitReceiveGen(const Value: TReceiveGenEvent);
    procedure SetBeforeCommitReceiveGen(const Value: TReceiveGenEvent);
  public
    { Public declarations }
    procedure Receive( const FileName: string );
    property  OnBeforeCommitReceiveGen: TReceiveGenEvent read FBeforeCommitReceiveGen write SetBeforeCommitReceiveGen;
    property  OnAfterCommitReceiveGen: TReceiveGenEvent read FAfterCommitReceiveGen write SetAfterCommitReceiveGen;
  end;

var
  InteractionDM: TInteractionDM;

implementation

uses unClientMainDM,  Controls, IB, Dialogs,
  unTools, unServerMainDM, Windows, unGlobal;

{$R *.dfm}

{ TRemoteDM }

(*********************************************************************)
(*     DB_PROFILE.                 *)
(*  DB_PROFILE_INPUT.                                 *)
(*   StartReceive.                              *)
(*********************************************************************)
procedure TInteractionDM.ReceiveDBID( Gen: Integer );
  const
    SQL: string =
      'insert into DB_PROFILE_INPUT(  LOG_DB_ID,  LOG_ACTION ) ' +
      '                     values(  :LOG_DB_ID, :LOG_ACTION )';
  begin
    Server_DBIDGeneration.ParamByName( 'CLIENT_DB_ID' ).AsInteger  := ClientMainDM.GetCurrDBID;
    Server_DBIDGeneration.ParamByName( 'SERV_GEN'    ).AsInteger := Gen;
    Server_DBIDGeneration.Open;
    try
      Server_DBIDGeneration.First;
      Client_Query.SQL.Text := SQL;
      while not Server_DBIDGeneration.Eof do
        begin
          Client_Query.ParamByName( 'LOG_DB_ID' ).AsInteger    := Server_DBIDGeneration.FieldByName( 'LOG_DB_ID' ).AsInteger;
          Client_Query.ParamByName( 'LOG_ACTION' ).AsString    := Server_DBIDGeneration.FieldByName( 'LOG_ACTION' ).AsString;
          Client_Query.ExecSQL;
          Server_DBIDGeneration.Next;
        end;
    finally
      Server_DBIDGeneration.Close;
    end;
  end;



(*********************************************************************)
(*     MASTER.                     *)
(*  MASTER_INPUT.                                    *)
(*   StartReceive.                              *)
(*********************************************************************)
procedure TInteractionDM.ReceiveMaster( Gen: Integer );
  const
    SQL: string =
      'insert into MASTER_INPUT( LOG_GID,  LOG_OWN_GEN,  LOG_ACTION,  DATA ) ' +
      '                 values(  :LOG_GID, :LOG_OWN_GEN, :LOG_ACTION, :DATA )';
  begin
    Server_MasterGeneration.ParamByName( 'CLIENT_DB_ID' ).AsInteger  := ClientMainDM.GetCurrDBID;
    Server_MasterGeneration.ParamByName( 'SERV_GEN'    ).AsInteger   := Gen;
    Server_MasterGeneration.Open;
    try
      Server_MasterGeneration.First;
      Client_Query.SQL.Text := SQL;
      while not Server_MasterGeneration.Eof do
        begin
          Client_Query.ParamByName( 'LOG_GID' ).AsString      := Server_MasterGeneration.FieldByName( 'LOG_GID' ).AsString;
          Client_Query.ParamByName( 'LOG_OWN_GEN' ).AsInteger := Server_MasterGeneration.FieldByName( 'LOG_OWN_GEN' ).AsInteger;
          Client_Query.ParamByName( 'LOG_ACTION' ).AsString   := Server_MasterGeneration.FieldByName( 'LOG_ACTION' ).AsString;
          (*Table specific*)
          Client_Query.ParamByName( 'DATA' ).AsString         := Server_MasterGeneration.FieldByName( 'DATA' ).AsString;
          (*/Table specific*)
          Client_Query.ExecSQL;
          Server_MasterGeneration.Next;
        end;
    finally
      Server_MasterGeneration.Close;
    end;
  end;

(*********************************************************************)
(*     DETAIL.                     *)
(*  DETAIL_INPUT.                                    *)
(*   StartReceive                               *)
(*********************************************************************)
procedure TInteractionDM.ReceiveDetail( Gen: Integer );
  const
    SQL: string =
      'insert into DETAIL_INPUT(  LOG_GID,  LOG_OWN_GEN,  LOG_ACTION,  DATA,  MASTER_GID ) ' +
      '                 values(  :LOG_GID, :LOG_OWN_GEN, :LOG_ACTION, :DATA, :MASTER_GID )';
  begin
    Server_DetailGeneration.ParamByName( 'CLIENT_DB_ID' ).AsInteger  := ClientMainDM.GetCurrDBID;
    Server_DetailGeneration.ParamByName( 'SERV_GEN'    ).AsInteger := Gen;
    Server_DetailGeneration.Open;
    try
      Server_DetailGeneration.First;
      Client_Query.SQL.Text := SQL;
      while not Server_DetailGeneration.Eof do
        begin
          Client_Query.ParamByName( 'LOG_GID' ).AsString      := Server_DetailGeneration.FieldByName( 'LOG_GID' ).AsString;
          Client_Query.ParamByName( 'LOG_OWN_GEN' ).AsInteger := Server_DetailGeneration.FieldByName( 'LOG_OWN_GEN' ).AsInteger;
          Client_Query.ParamByName( 'LOG_ACTION' ).AsString   := Server_DetailGeneration.FieldByName( 'LOG_ACTION' ).AsString;
          (*Table specific*)
          Client_Query.ParamByName( 'DATA' ).AsString         := Server_DetailGeneration.FieldByName( 'DATA' ).AsString;
          Client_Query.ParamByName( 'MASTER_GID' ).AsString   := Server_DetailGeneration.FieldByName( 'MASTER_GID' ).AsString;
          (*/Table specific*)
          Client_Query.ExecSQL;
          Server_DetailGeneration.Next;
        end;
    finally
      Server_DetailGeneration.Close;
    end;
  end;

(*********************************************************************)
(*    FileName                                 *)
(*   EAbort                                 *)
(*********************************************************************)
procedure TInteractionDM.Receive(const FileName: string);
  var
    Server_CurrGen:   Integer;
    Client_ServGen:   Integer;
    gen:              Integer;
  begin
    ServerMainDM.Open( FileName );
    //,     ClientMainDM.Open
    try
      //    ,      (IS_CURR = 1)
      //  DB_PROFILE
      if ClientMainDM.Deleted then
       begin
         ShowMessage( 'DB is deleted' );
         ABORT;
       end;

      Server_CurrGen   := ServerMainDM.GetCurrGen;//   .  CURR_GEN     
      Client_ServGen   := ClientMainDM.GetServGen( ServerMainDM.GetCurrDBID );//      . Select  DB_PROFILE     
      // ,     Client_ServGen
      //( ,         ,
      //        (   ) )
      //      
      // SET_CLIENT_GEN  
      ServerMainDM.SetClientGen( ClientMainDM.GetCurrDBID, Client_ServGen );
      for gen := Client_ServGen + 1 to Server_CurrGen do
        begin
          StartReceive( gen );//   gen.  BEGIN_RECEIVE
          try
            ReceiveDBID( gen );//   DB_PROFILE ( DB_PROFILE_INPUT).  DB_ID_GEN  
            (******Scema specific*****)
            ReceiveMaster( gen );//   MASTER ( MASTER_INPUT).  MASTER_GEN  
            ReceiveDetail( gen );//   DETAIL ( DETAIL_INPUT).  DETAIL_GEN  
            (******/Scema specific****)
            ApplyReceivedData;// (  XXX_INPUT).  PROCESS_INPUT_TABLES
            if Assigned( OnBeforeCommitReceiveGen ) then
              OnBeforeCommitReceiveGen( Self, gen );

            CommitReceive;//   ( ).  END_RECEIVE

            //       ( SET_NEW_GEN  )
            //       post  Backgrounds.exe 
            if Assigned( OnAfterCommitReceiveGen ) then
              OnAfterCommitReceiveGen( Self, gen );
          except
            RollbackReceive;//   ( )
            raise;
          end;
          // ,     gen
          //      
          //      .
          // SET_CLIENT_GEN  
          ServerMainDM.SetClientGen( ClientMainDM.GetCurrDBID, gen );
        end;(*for*)
    finally
      ServerMainDM.Close;
    end;(*try*)
  end;

(**********************************************************************)
(* (  XXX_INPUT)                 *)
(*   StartReceive                                *)
(**********************************************************************)
procedure TInteractionDM.ApplyReceivedData;
  begin
    Client_ProcessInputTablesProc.ExecProc;
  end;

(*********************************************************************)
(*  .                                       *)
(*  ,      ,  *)
(*   (  StartReceive),     *)
(* .                                           *)
(*********************************************************************)
procedure TInteractionDM.CommitReceive;
  begin
    Client_EndReceiveProc.ExecProc;
    Client_Transaction.Commit;
    Server_Transaction.Commit;
  end;


(*********************************************************************)
(*  .                                              *)
(*     .                             *)
(* .                                             *)
(*********************************************************************)
procedure TInteractionDM.RollbackReceive;
  begin
    Client_Transaction.Rollback;
    Server_Transaction.Rollback;
  end;

(*********************************************************************)
(*  .                                               *)
(*     .                            *)
(*                                             *)
(*    CommitReceive  RollbackReceive.    *)
(*                                                 *)
(*SERV_GEN -    ,      *)
(*********************************************************************)
procedure TInteractionDM.StartReceive( SERV_GEN: Integer );
  begin
    try
      Server_Transaction.StartTransaction;
      Client_Transaction.StartTransaction;

      Client_BeginReceiveProc.ParamByName( 'A_SERV_ID' ).AsInteger := ServerMainDM.GetCurrDBID;
      Client_BeginReceiveProc.ParamByName( 'A_SERV_GEN' ).AsInteger   := SERV_GEN;
      Client_BeginReceiveProc.ExecProc;
    except
      Client_Transaction.Rollback;
      Server_Transaction.Rollback;
      raise;
    end;
  end;

procedure TInteractionDM.Client_MasterInputLOG_GIDGetText(Sender: TField;
  var Text: String; DisplayText: Boolean);
  begin
    if DisplayText then
      Text := GIDToStr( Sender.AsVariant );
  end;

procedure TInteractionDM.Client_DetailInputLOG_GIDGetText(Sender: TField;
  var Text: String; DisplayText: Boolean);
  begin
    if DisplayText then
      Text := GIDToStr( Sender.AsVariant );
  end;

procedure TInteractionDM.SetAfterCommitReceiveGen(
  const Value: TReceiveGenEvent);
begin
  FAfterCommitReceiveGen := Value;
end;

procedure TInteractionDM.SetBeforeCommitReceiveGen(
  const Value: TReceiveGenEvent);
  begin
    FBeforeCommitReceiveGen := Value;
  end;

end.
