(*********************************************************)
(* TServerMainDM                 *)
(*      			         *)
(*    			 *)
(*  ServerMainDM.Database          *)
(*    Database                        *)
(*  DataModule     *)
(*********************************************************)
unit unServerMainDM;

interface

uses
  SysUtils, Classes, IBStoredProc, DB, IBCustomDataSet, IBQuery, IBDatabase;

type
  TServerMainDM = class(TDataModule)
    Database: TIBDatabase;
    WriteTransaction: TIBTransaction;
    ReadQuery: TIBQuery;
    SetClientGenProc: TIBStoredProc;
    ReadTransaction: TIBTransaction;
  private
    { Private declarations }
  public
    { Public declarations }
    function   GetServGen(DB_ID: Integer): Integer;(*     DB_ID(     DB_ID)*)
    function   GetCurrDBID: Integer;
    function   GetCurrGen: Integer;
    procedure  SetClientGen(A_CLIENT_DB_ID, A_CLIENT_SERV_GEN: Integer);(*    A_CLIENT_DB_ID(    )    *)
    procedure  Open( DatabaseName: string );
    procedure  Close;
  end;

var
  ServerMainDM: TServerMainDM;

implementation

{$R *.dfm}


procedure TServerMainDM.SetClientGen( A_CLIENT_DB_ID, A_CLIENT_SERV_GEN: Integer );
  begin
    WriteTransaction.StartTransaction;
    try
      SetClientGenProc.ParamByName( 'A_CLIENT_DB_ID' ).AsInteger := A_CLIENT_DB_ID;
      SetClientGenProc.ParamByName( 'A_CLIENT_SERV_GEN' ).AsInteger := A_CLIENT_SERV_GEN;
      SetClientGenProc.ExecProc;
      WriteTransaction.Commit;
    except
      WriteTransaction.Rollback;
      raise;
    end;
  end;



function TServerMainDM.GetCurrDBID: Integer;
  const
    SQL: string = 'select DB_ID from CURR_DB_ID';
  begin
    ReadQuery.SQL.Text := 'select DB_ID from CURR_DB_ID';
    ReadTransaction.StartTransaction;
    try
      ReadQuery.Open;
      try
        ReadQuery.First;
        Result := ReadQuery.FieldByName( 'DB_ID' ).AsInteger;
        ReadQuery.Next;
        if not ReadQuery.Eof then
            raise Exception.Create( 'TServerMainDM.GetCurrDBID' );
      finally
        ReadQuery.Close;
      end;
      ReadTransaction.Commit;
    except
      ReadTransaction.Rollback;
      raise;
    end;
  end;



function TServerMainDM.GetCurrGen: Integer;
  const
    SQL: string = 'select GEN from CURR_GEN';
  begin
    ReadQuery.SQL.Text := SQL;
    ReadTransaction.StartTransaction;
    try
      ReadQuery.Open;
      try
        ReadQuery.First;
        Result := ReadQuery.FieldByName( 'GEN' ).AsInteger;
        ReadQuery.Next;
        if not ReadQuery.Eof then
           raise Exception.Create( 'TServerMainDM.GetCurrGen' );
      finally
        ReadQuery.Close;
      end;
      ReadTransaction.Commit;
    except
      ReadTransaction.Rollback;
      raise;
    end;
  end;


function TServerMainDM.GetServGen( DB_ID: Integer ): Integer;
  const
    SQL: string = 'SELECT SERV_GEN FROM DB_PROFILE WHERE  DB_ID = :DB_ID';
  begin
    ReadTransaction.StartTransaction;
    try
      ReadQuery.SQL.Text := SQL;
      ReadQuery.ParamByName( 'DB_ID' ).AsInteger := DB_ID;
      ReadQuery.Open;
      try
        ReadQuery.First;
        Result := ReadQuery.FieldByName( 'SERV_GEN' ).AsInteger;
        ReadQuery.Next;
        if not ReadQuery.Eof then
           raise Exception.Create( 'TServerMainDM.GetServGen' );
      finally
        ReadQuery.Close;
      end;
      ReadTransaction.Commit;
    except
      ReadTransaction.Rollback;
      raise;
    end;
  end;



procedure TServerMainDM.Open( DatabaseName: string );
  begin
    Database.Connected := False;
    Database.DatabaseName := DatabaseName;
    Database.Open;
  end;

procedure TServerMainDM.Close;
  begin
    Database.Close;
  end;

end.
