unit unTabFrame;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, Spin;

type
  TLogEvent = procedure( Sender: TObject; const LogStr: string ) of object;

  TTabFrame = class(TFrame)
    lbDatabaseName: TLabel;
    cbxActive: TCheckBox;
    sedInterval: TSpinEdit;
    lbInterval: TLabel;
    Timer: TTimer;
    btForceReplication: TButton;
    procedure cbxActiveClick(Sender: TObject);
    procedure sedIntervalChange(Sender: TObject);
    procedure TimerTimer(Sender: TObject);
    procedure btForceReplicationClick(Sender: TObject);
  private
    { Private declarations }
    FOnLog: TLogEvent;
    FDatabaseName: string;
    FDBID: Integer;
    procedure AddLog( const LogStr: string );
    procedure AddLogFmt( const FormatStr: string; const Args: array of TVarRec );
    procedure SetDatabaseName(const Value: string);
  public
    { Public declarations }
    property OnLog: TLogEvent read FOnLog write FOnLog;
    property DatabaseName: string read FDatabaseName write SetDatabaseName;
    property DBID: Integer read FDBID write FDBID;
  end;

implementation

uses unInteractionDM;

{$R *.dfm}


procedure TTabFrame.AddLog( const LogStr: string );
  begin
    if Assigned( OnLog ) then
      OnLog( self, LogStr );
  end;

procedure TTabFrame.AddLogFmt( const FormatStr: string; const Args: array of TVarRec );
  begin
    AddLog( Format( FormatStr, Args ) );
  end;

procedure TTabFrame.cbxActiveClick(Sender: TObject);
  begin
    Timer.Enabled := cbxActive.Checked;
  end;

procedure TTabFrame.sedIntervalChange(Sender: TObject);
  begin
    Timer.Interval := sedInterval.Value * 1000;
  end;

procedure TTabFrame.SetDatabaseName(const Value: string);
  begin
    FDatabaseName := Value;
    lbDatabaseName.Caption := FDatabaseName;
  end;

procedure TTabFrame.TimerTimer(Sender: TObject);
  var
    Buf: Boolean;
  begin
    Buf := Timer.Enabled;
    Timer.Enabled := False;
    AddLogFmt( '%d begin receive', [ DBID ] );
    InteractionDM.Receive( DatabaseName );
    AddLogFmt( '%d end receive', [ DBID ] );
    Timer.Enabled := Buf;
  end;

procedure TTabFrame.btForceReplicationClick(Sender: TObject);
  var
    Buf: Boolean;
  begin
    Buf := Timer.Enabled;
    Timer.Enabled := False;
    AddLogFmt( '%d begin receive (force replication)', [ DBID ] );
    InteractionDM.Receive( DatabaseName );
    AddLogFmt( '%d end receive', [ DBID ] );
    Timer.Enabled := Buf;
  end;

end.
